/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDInput.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDInput::VDInput() {
    directInput = NULL;
    keyboard = NULL;
    mouse = NULL;
    SecureZeroMemory(keys, sizeof(BYTE) * 256);
    SecureZeroMemory(&mouseState, sizeof(DIMOUSESTATE));
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDInput::~VDInput() {
    if (keyboard != NULL) {
        (void) keyboard->Unacquire();
        (void) keyboard->Release();
        keyboard = NULL;
    };
    if (mouse != NULL) {
        (void) mouse->Unacquire();
        (void) mouse->Release();
        mouse = NULL;
    };
    if (directInput != NULL) {
        (void) directInput->Release();
        directInput = NULL;
    };
};



void VDInput::InitializeInput(HINSTANCE instance) {
    (void) DirectInput8Create(instance, DIRECTINPUT_VERSION, IID_IDirectInput8, (void **) &directInput, NULL);
};


void VDInput::InitializeKeyboard(HWND window) {
    (void) directInput->CreateDevice(GUID_SysKeyboard, &keyboard, NULL);
    (void) keyboard->SetDataFormat(&c_dfDIKeyboard);
    (void) keyboard->SetCooperativeLevel(window, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE);
    (void) keyboard->Acquire();
};

void VDInput::InitializeMouse(HWND window) {
    (void) directInput->CreateDevice(GUID_SysMouse, &mouse, NULL);
    (void) mouse->SetDataFormat(&c_dfDIMouse);
    (void) mouse->SetCooperativeLevel(window, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE);
    (void) mouse->Acquire();
};

void VDInput::RetrieveLostInput() {
    if (keyboard != NULL) {
        if (keyboard->Poll() == DIERR_INPUTLOST || keyboard->Poll() == DIERR_NOTACQUIRED) {
            (void) keyboard->Acquire();
        };
    };
    if (mouse != NULL) {
        if (mouse->Poll() == DIERR_INPUTLOST || mouse->Poll() == DIERR_NOTACQUIRED) {
            (void) mouse->Acquire();
        };
    };
};

void VDInput::UpdateInput() {
    if (keyboard != NULL) {
        (void) keyboard->GetDeviceState(sizeof(BYTE[256]), (LPVOID) keys);
    };
    if (mouse != NULL) {
        (void) mouse->GetDeviceState(sizeof(DIMOUSESTATE), (LPVOID) &mouseState);
    };
};



/* **********************************************************************************************************************
   Accessors and Modifiers
   **********************************************************************************************************************/

BYTE *VDInput::GetKeyboardInput() {
    return keys;
};

DIMOUSESTATE VDInput::GetMouseInput() {
    return mouseState;
};