/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDMapEditor.h"



/* **********************************************************************************************************************
   Global Function Prototypes
   **********************************************************************************************************************/

LRESULT CALLBACK WindowProcedure(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDMapEditor::VDMapEditor() {
    window = NULL;
    mapNameEditBox = NULL;
    saveMapButton = NULL;
    loadMapButton = NULL;
    loadSpriteButton = NULL;
    tileSprite = NULL;
    (void) SecureZeroMemory(&tileSpriteDimensions, sizeof(tileSpriteDimensions));
    nullTileOption = NULL;
    openTileOption = NULL;
    impassableTileOption = NULL;
    newAreaTileOption = NULL;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDMapEditor::~VDMapEditor() {
    if (tileSprite != NULL) {
        delete tileSprite;
        tileSprite = NULL;
    };
};



/* **********************************************************************************************************************
   CreateMapEditor
   ---------------
   Creates a new Map Editor window.
   #param  instance   The instance of the application.
   #param  className  The name of the window class.
   #return  Handle to the window.
   **********************************************************************************************************************/

HWND VDMapEditor::CreateMapEditor(HINSTANCE instance, LPCSTR className) {
    window = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,      // extended window styles
                            className,                             // class name
                            "Map Editor",                          // window name
                            WS_CAPTION | WS_SYSMENU | WS_VISIBLE,  // window styles
                            100,                                   // x coordinate (0 = left)
                            100,                                   // y coordinate (0 = top)
                            170,                                   // width
                            400,                                   // height
                            NULL,                                  // parent/owner window
                            NULL,                                  // menu
                            instance,                              // instance
                            NULL);                                 // used for CREATESTRUCT structure
    if (window == NULL) {
        VDError("Failed to create the map editor window.", __FILE__, __LINE__);
    };

    mapNameEditBox = CreateWindow("EDIT", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 10, 10, 140, 18, window, (HMENU) VD_MAP_EDITOR_MAP_NAME, instance, NULL);
    if (mapNameEditBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    saveMapButton = CreateWindow("BUTTON", "Save Map", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 36, 70, 20, window, (HMENU) VD_MAP_EDITOR_SAVE_MAP, instance, NULL);
    if (saveMapButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // TODO: change from strlen to hard coded
    loadMapButton = CreateWindow("BUTTON", "Load Map", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 85, 36, 70, 20, window, (HMENU) VD_MAP_EDITOR_LOAD_MAP, instance, NULL);
    if (loadMapButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND divider = CreateWindow("STATIC", "-----------------------------------", WS_CHILD | WS_VISIBLE, 10, 56, 140, 10, window, (HMENU) VD_MAP_EDITOR_DIVIDER, instance, NULL);
    if (divider == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadSpriteButton = CreateWindow("BUTTON", "Load Sprite", WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON, 10, 70, 90, 20, window, (HMENU) VD_MAP_EDITOR_LOAD_SPRITE, instance, NULL);
    if (loadSpriteButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    tileSprite = NULL;
    tileSpriteDimensions.left = 112;
    tileSpriteDimensions.top = 70;
    tileSpriteDimensions.right = 144;
    tileSpriteDimensions.bottom = 102;
    nullTileOption = CreateWindow("button", "Null", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON | WS_GROUP, 10, 100, strlen("Null") * 20, 24, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_NULL, instance, NULL);
    if (nullTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    (void) SendMessage(nullTileOption, BM_SETCHECK, BST_CHECKED, 0);
    openTileOption = CreateWindow("BUTTON", "Open", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 124, strlen("Open") * 20, 24, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_OPEN, instance, NULL);
    if (openTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    impassableTileOption = CreateWindow("BUTTON", "Impassable", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 148, strlen("Impassable") * 20, 24, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_IMPASSABLE, instance, NULL);
    if (impassableTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    newAreaTileOption = CreateWindow("BUTTON", "New Area", WS_CHILD | WS_VISIBLE | BS_AUTORADIOBUTTON, 10, 172, strlen("New Area") * 20, 24, window, (HMENU) VD_MAP_EDITOR_TILE_OPTION_NEW_AREA, instance, NULL);
    if (newAreaTileOption == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    return window;
};



/* **********************************************************************************************************************
   Accessors and Modifiers
   **********************************************************************************************************************/

HWND VDMapEditor::GetWindow() {
    return window;
};

HWND VDMapEditor::GetMapNameEditBox() {
    return mapNameEditBox;
};

HWND VDMapEditor::GetSaveMapButton() {
    return saveMapButton;
};

HWND VDMapEditor::GetLoadMapButton() {
    return loadMapButton;
};

HWND VDMapEditor::GetLoadSpriteButton() {
    return loadSpriteButton;
};

VDSprite* VDMapEditor::GetTileSprite() {
    return tileSprite;
};

void VDMapEditor::SetTileSprite(VDSprite *newSprite) {
    if (tileSprite != NULL) {
        delete tileSprite;
        tileSprite = NULL;
    };
    tileSprite = newSprite;
};

LPRECT VDMapEditor::GetTileSpriteDimensions(LPRECT outRect) {
    outRect->left = tileSpriteDimensions.left;
    outRect->top = tileSpriteDimensions.top;
    outRect->right = tileSpriteDimensions.right;
    outRect->bottom = tileSpriteDimensions.bottom;

    return &tileSpriteDimensions;
};

HWND VDMapEditor::GetNullTileOption() {
    return nullTileOption;
};

HWND VDMapEditor::GetOpenTileOption() {
    return openTileOption;
};

HWND VDMapEditor::GetImpassableTileOption() {
    return impassableTileOption;
};

HWND VDMapEditor::GetNewAreaTileOption() {
    return newAreaTileOption;
};