/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDNPC.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDNPC::VDNPC() {
    (void) dialogues.empty();
};

VDNPC::VDNPC(LPCSTR aName) {
    (void) strcpy_s(name, aName);
    (void) dialogues.empty();
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDNPC::~VDNPC() {
};



/* **********************************************************************************************************************
   Move
   ----
   Moves the NPC in a random direction.
   **********************************************************************************************************************/

void VDNPC::Move() {
    int direction = 0;

    // randomize movement
    direction = rand() % 5;

    // move the NPC
    switch (direction) {
        case 0:
            currentLocationX += 0;
            currentLocationY += 0;
            break;
        case 1:
            currentLocationX -= 1;
            break;
        case 2:
            currentLocationX += 1;
            break;
        case 3:
            currentLocationY -= 1;
            break;
        case 4:
            currentLocationY += 1;
            break;
        default:
            currentLocationX += 0;
            currentLocationY += 0;
            break;
    };
};



/* **********************************************************************************************************************
   AddDialogue
   -----------
   Adds a new dialogue to the list.
   #param  id    The ID of the dialogue.
   #param  text  The text of the dialogue.
   #return  TRUE if successful, FALSE if the ID already exists.
   **********************************************************************************************************************/

int VDNPC::AddDialogue(int id, LPCSTR text) {
    // look through all the dialogues
    for (unsigned int n = 0; n < dialogues.size(); n++) {
        // if one with this ID exists, do nothing
        if (dialogues.at(n).GetID() == id) {
            dialogues.at(n).SetText(text);
            return FALSE;
        };
    };

    // add the new dialogue
    VDDialogue dialogue = VDDialogue(id, text);
    dialogues.push_back(dialogue);

    return TRUE;
};

// TODO
void VDNPC::RemoveDialogue(unsigned int id) {
    for (unsigned int n = 0; n < dialogues.size(); n++) {
        if (dialogues.at(n).GetID() == id) {
            dialogues.erase(dialogues.begin() + n);
        };
    };
};



/* **********************************************************************************************************************
   Accessors and Modifiers
   **********************************************************************************************************************/

int VDNPC::GetDialoguesSize() {
    return dialogues.size();
};

LPCSTR VDNPC::GetDialogue(unsigned int index) {
    return dialogues.at(index).GetText();
};

LPCSTR VDNPC::GetDialogueFromID(int id) {
    for (unsigned int n = 0; n < dialogues.size(); n++) {
        if (dialogues.at(n).GetID() == id) {
            return dialogues.at(n).GetText();
        };
    };
};

unsigned int VDNPC::GetDialogueID(unsigned int index) {
    return dialogues.at(index).GetID();
};