/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDNPCEditor.h"



/* **********************************************************************************************************************
   Global Function Prototypes
   **********************************************************************************************************************/

LRESULT CALLBACK WindowProcedure(HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDNPCEditor::VDNPCEditor() {
    window = NULL;
    npcListBox = NULL;
    nameEditBox = NULL;
    loadSpriteButton = NULL;
    npcSprite = NULL;
    (void) SecureZeroMemory(&npcSpriteLocation, sizeof(npcSpriteLocation));
    (void) SecureZeroMemory(&npcSpriteDimensions, sizeof(npcSpriteDimensions));
    removeNPCButton = NULL;
    saveNPCButton = NULL;
    dialogueListBox = NULL;
    textEditBox = NULL;
    textIDEditBox = NULL;
    removeTextButton = NULL;
    saveTextButton = NULL;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDNPCEditor::~VDNPCEditor() {
    if (npcSprite != NULL) {
        delete npcSprite;
        npcSprite = NULL;
    };
};



/* **********************************************************************************************************************
   CreateNPCEditor
   ---------------
   Creates a new NPC Editor window.
   #param  instance   The instance of the application.
   #param  className  The name of the window class.
   #param  npcList    List of NPC's to populate the list box.
   #return  Handle to the window.
   **********************************************************************************************************************/

HWND VDNPCEditor::CreateNPCEditor(HINSTANCE instance, LPCSTR className, list<VDNPC> npcList) {
    window = CreateWindowEx(WS_EX_TOPMOST | WS_EX_TOOLWINDOW,      // extended window styles
                            className,                             // class name
                            "NPC Editor",                          // window name
                            WS_CAPTION | WS_SYSMENU | WS_VISIBLE,  // window styles
                            200,                                   // x coordinate (0 = left)
                            200,                                   // y coordinate (0 = top)
                            500,                                   // width
                            600,                                   // height
                            NULL,                                  // parent/owner window
                            NULL,                                  // menu
                            instance,                              // instance
                            NULL);                                 // used for CREATESTRUCT structure
    if (window == NULL) {
        VDError("Failed to create the window.", __FILE__, __LINE__);
    };

    // NPC's
    HWND npcListStatic = CreateWindow("Static", "NPC's:", WS_CHILD | WS_VISIBLE, 10, 10, 40, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_NPC_LIST, instance, NULL);
    if (npcListStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    npcListBox = CreateWindow("ListBox", NULL, WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 30, 100, 200, window, (HMENU) VD_NPC_EDITOR_NPC_LIST, instance, NULL);
    if (npcListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND npcNameStatic = CreateWindow("Static", "Name:", WS_CHILD | WS_VISIBLE, 120, 30, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_NAME, instance, NULL);
    if (npcNameStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    nameEditBox = CreateWindow("Edit", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 180, 30, 140, 18, window, (HMENU) VD_NPC_EDITOR_NAME, instance, NULL);
    if (nameEditBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    loadSpriteButton = CreateWindow("Button", "Load Sprite", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 120, 80, 90, 20, window, (HMENU) VD_NPC_EDITOR_LOAD_SPRITE, instance, NULL);
    if (loadSpriteButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    npcSprite = NULL;
    npcSpriteDimensions.left = 225;
    npcSpriteDimensions.top = 68;
    npcSpriteDimensions.right = 257;
    npcSpriteDimensions.bottom = 100;
    removeNPCButton = CreateWindow("Button", "Remove NPC", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 230, 100, 20, window, (HMENU) VD_NPC_EDITOR_LOAD_SPRITE, instance, NULL);
    if (removeNPCButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    saveNPCButton = CreateWindow("Button", "Save NPC", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 120, 107, 80, 20, window, (HMENU) VD_NPC_EDITOR_LOAD_SPRITE, instance, NULL);
    if (saveNPCButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // NPC dialogue
    HWND dialogueStatic = CreateWindow("Static", "Dialogue:", WS_CHILD | WS_VISIBLE, 10, 260, 70, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_DIALOGUE, instance, NULL);
    if (dialogueStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    dialogueListBox = CreateWindow("ListBox", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL, 10, 280, 200, 110, window, (HMENU) VD_NPC_EDITOR_DIALOGUE, instance, NULL);
    if (dialogueListBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND textStatic = CreateWindow("Static", "Text:", WS_CHILD | WS_VISIBLE, 220, 260, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_TEXT, instance, NULL);
    if (textStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    textEditBox = CreateWindow("Edit", "", WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | ES_MULTILINE, 220, 280, 200, 60, window, (HMENU) VD_NPC_EDITOR_TEXT, instance, NULL);
    if (textEditBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    HWND textIDStatic = CreateWindow("Static", "Text ID:", WS_CHILD | WS_VISIBLE, 220, 345, 50, 18, window, (HMENU) VD_NPC_EDITOR_STATIC_TEXT_ID, instance, NULL);
    if (textIDStatic == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    textIDEditBox = CreateWindow("Edit", "", WS_CHILD | WS_BORDER | WS_VISIBLE, 275, 345, 30, 18, window, (HMENU) VD_NPC_EDITOR_TEXT_ID, instance, NULL);
    if (textIDEditBox == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    removeTextButton = CreateWindow("Button", "Remove Text", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 10, 385, 100, 20, window, (HMENU) VD_NPC_EDITOR_REMOVE_TEXT, instance, NULL);
    if (removeTextButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };
    saveTextButton = CreateWindow("Button", "Save Text", WS_CHILD | BS_PUSHBUTTON | WS_VISIBLE, 220, 370, 80, 20, window, (HMENU) VD_NPC_EDITOR_SAVE_TEXT, instance, NULL);
    if (saveTextButton == NULL) {
        VDError("Failed to create an interface resource.", __FILE__, __LINE__);
    };

    // look at the list of NPC's and add them to the NPC list box
    for (list<VDNPC>::iterator anNPC = npcList.begin(); anNPC != npcList.end(); ++anNPC) {
        VDNPC currentNPC = * anNPC;
        (void) SendMessage(npcListBox, LB_ADDSTRING, 0, (LPARAM) currentNPC.GetName());
    };

    return window;
};



/* **********************************************************************************************************************
   Accessors and Modifiers
   **********************************************************************************************************************/

HWND VDNPCEditor::GetWindow() {
    return window;
};

HWND VDNPCEditor::GetNPCListBox() {
    return npcListBox;
};

HWND VDNPCEditor::GetNameEditBox() {
    return nameEditBox;
};

HWND VDNPCEditor::GetLoadSpriteButton() {
    return loadSpriteButton;
};

VDSprite* VDNPCEditor::GetNPCSprite() {
    return npcSprite;
};

void VDNPCEditor::SetNPCSprite(VDSprite *newSprite) {
    if (npcSprite != NULL) {
        delete npcSprite;
        npcSprite = NULL;
    };
    npcSprite = newSprite;
};

LPCSTR VDNPCEditor::GetNPCSpriteLocation() {
    return npcSpriteLocation;
};

void VDNPCEditor::SetNPCSpriteLocation(LPCSTR spriteLocation) {
    (void) strcpy_s(npcSpriteLocation, spriteLocation);
};

LPRECT VDNPCEditor::GetNPCSpriteDimensions(LPRECT outRect) {
    outRect->left = npcSpriteDimensions.left;
    outRect->top = npcSpriteDimensions.top;
    outRect->right = npcSpriteDimensions.right;
    outRect->bottom = npcSpriteDimensions.bottom;

    return &npcSpriteDimensions;
};

HWND VDNPCEditor::GetRemoveNPCButton() {
    return removeNPCButton;
};

HWND VDNPCEditor::GetSaveNPCButton() {
    return saveNPCButton;
};

HWND VDNPCEditor::GetDialogueListBox() {
    return dialogueListBox;
};

HWND VDNPCEditor::GetTextEditBox() {
    return textEditBox;
};

HWND VDNPCEditor::GetTextIDEditBox() {
    return textIDEditBox;
};

HWND VDNPCEditor::GetRemoveTextButton() {
    return removeTextButton;
};

HWND VDNPCEditor::GetSaveTextButton() {
    return saveTextButton;
};