/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDSprite.h"




/* **********************************************************************************************************************
   Constructors
   ------------
   #param  direct3DDevice  Pointer to the display adapter device.
   #param  textureFile     Name of the texture (with directory structure included).
   #param  x               Width of the sprite.
   #param  y               Height of the sprite.
   #param  aCenter         Center of the sprite.
   #param  aPosition       Position of the sprite on the screen.
   #param  aColor          Color of the sprite.
   #param  isStaticSprite  Is the sprite static?
   #param  isNPCSprite     Is the sprite an NPC?
   **********************************************************************************************************************/

VDSprite::VDSprite() {
    texture = NULL;
    SecureZeroMemory(&sourceRectangle, sizeof(sourceRectangle));
    SecureZeroMemory(&center, sizeof(center));
    SecureZeroMemory(&position, sizeof(position));
    color = NULL;
    isStatic = FALSE;
    isNPC = FALSE;
    isDead = FALSE;
};

VDSprite::VDSprite(LPDIRECT3DDEVICE9 direct3DDevice, LPCSTR textureFile, int x, int y, D3DXVECTOR3 aCenter, D3DXVECTOR3 aPosition, D3DCOLOR aColor, int isStaticSprite, int isNPCSprite) {
    HRESULT returnValue = NULL;

    // uses 0xFFFFFFFF for the color key, which "removes" white from the texture and makes those areas fully transparent
    if (isStaticSprite == TRUE || isNPCSprite == TRUE) {
        returnValue = D3DXCreateTextureFromFileEx(direct3DDevice, textureFile, x, y, D3DX_DEFAULT, NULL, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0xFFFFFFFF, NULL, NULL, &texture);
        if (returnValue != D3D_OK) {
            VDDirectXError(textureFile, returnValue, __FILE__, __LINE__);
        };
    // uses 0xFF000000 for the color key, which keeps the texture the same
    } else {
        returnValue = D3DXCreateTextureFromFileEx(direct3DDevice, textureFile, x, y, D3DX_DEFAULT, NULL, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, D3DX_DEFAULT, D3DX_DEFAULT, 0x00000000, NULL, NULL, &texture);
        if (returnValue != D3D_OK) {
            VDDirectXError(textureFile, returnValue, __FILE__, __LINE__);
        };
    };
    SetRect(&sourceRectangle, 0, 0, x, y);
    center = aCenter;
    position = aPosition;
    color = aColor;
    isStatic = isStaticSprite;
    isNPC = isNPCSprite;
    isDead = FALSE;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDSprite::~VDSprite() {
    /*if (texture != NULL) {
        texture->Release();
        texture = NULL;
    };*/
};



/* **********************************************************************************************************************
   Draw
   ----
   Draws the sprite.
   #param  sprite  Pointer to the sprite interface.
   **********************************************************************************************************************/

void VDSprite::Draw(LPD3DXSPRITE sprite) {
    HRESULT returnValue = NULL;

    returnValue = sprite->Draw(texture, &sourceRectangle, &center, &position, color);
    if (returnValue != S_OK) {
        VDDirectXError("Failed to draw the sprite.", returnValue, __FILE__, __LINE__);
    };
};



/* **********************************************************************************************************************
   ShiftLocation
   -------------
   Translates a sprite.
   #param  direct3DDevice  Pointer to the Direct3D interface.
   #param  x               X coordinate of where to translate.
   #param  y               Y coordinate of where to translate.
   **********************************************************************************************************************/

void VDSprite::ShiftLocation(LPDIRECT3DDEVICE9 direct3DDevice, float x, float y) {
    D3DXMATRIX m_Translation;
    HRESULT returnValue = NULL;

    // set up the translation matrix
    (void) D3DXMatrixTranslation(&m_Translation, x, y, 0.0f);

    // apply the translation to the world
    returnValue = direct3DDevice->SetTransform(D3DTS_WORLD, &m_Translation);
    if (returnValue != D3D_OK) {
        VDDirectXError("Failed to set a single device transformation-related state.", returnValue, __FILE__, __LINE__);
    };
};



/* **********************************************************************************************************************
   Accessors and Modifiers
   **********************************************************************************************************************/

D3DXVECTOR3 VDSprite::GetPosition() {
    return position;
};

int VDSprite::IsStatic() {
    return isStatic;
};

int VDSprite::IsNPC() {
    return isNPC;
};

int VDSprite::IsDead() {
    return isDead;
};

void VDSprite::AddToPosition(int x, int y) {
    position.x += x;
    position.y += y;
};

void VDSprite::KillSprite() {
    isDead = TRUE;
};