/* **********************************************************************************************************************
   Included Header Files
   **********************************************************************************************************************/

#include "VDWorld.h"
#include "VDError.h"



/* **********************************************************************************************************************
   Constructor
   **********************************************************************************************************************/

VDWorld::VDWorld() {
    world = NULL;
    worldSizeX = 0;
    worldSizeY = 0;
};



/* **********************************************************************************************************************
   Destructor
   **********************************************************************************************************************/

VDWorld::~VDWorld() {
    for (int x = 0; x < worldSizeX; x++) {
        delete[] world[x];
        world[x] = NULL;
    };
    if (world != NULL) {
        delete[] world;
        world = NULL;
    };
};



/* **********************************************************************************************************************
   WriteWorld
   ----------
   Saves the current world to hard disk.
   #param  fileName  The name of the world file.
   **********************************************************************************************************************/

void VDWorld::CreateWorld(LPCSTR fileName) {
    DWORD numberOfBytesToWrite = 0;
    DWORD numberOfBytesWritten = 0;
    HANDLE file = NULL;
    BOOL writeFile = FALSE;
    BOOL closeFile = FALSE;

    numberOfBytesToWrite += worldSizeX;
    numberOfBytesToWrite += worldSizeY;
    for (int x = 0; x < worldSizeX; x++) {
        for (int y = 0; y < worldSizeY; y++) {
            numberOfBytesToWrite += sizeof(world[x][y]);
        };
    };

    // TODO: REMOVE
    /*char boom[100];
    sprintf_s(boom, "size of world: %u\n", numberOfBytesToWrite);
    MessageBox(NULL, boom, "Here", MB_OK);*/

    file = CreateFile(fileName, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    writeFile = WriteFile(file, &worldSizeX, sizeof(worldSizeX), &numberOfBytesWritten, NULL);
    writeFile = WriteFile(file, &worldSizeY, sizeof(worldSizeY), &numberOfBytesWritten, NULL);
    for (int x = 0; x < worldSizeX; x++) {
        for (int y = 0; y < worldSizeY; y++) {
            writeFile = WriteFile(file, &world[x][y], sizeof(world[x][y]), &numberOfBytesWritten, NULL);
        };
    };
    closeFile = CloseHandle(file);
};



/* **********************************************************************************************************************
   LoadWorld
   ---------
   Loads a world into the game.
   #param  fileName  The name of the world file.
   **********************************************************************************************************************/

void VDWorld::LoadWorld(LPCSTR fileName) {
    DWORD numberOfBytesRead = 0;
    HANDLE file = NULL;
    BOOL readFile = FALSE;
    BOOL closeFile = FALSE;

    // load file
    file = CreateFile(fileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if (file == INVALID_HANDLE_VALUE) {
        VDError("Failed to load the world file.", __FILE__, __LINE__);
    };
    readFile = ReadFile(file, &worldSizeX, sizeof(worldSizeX), &numberOfBytesRead, NULL);
    readFile = ReadFile(file, &worldSizeY, sizeof(worldSizeY), &numberOfBytesRead, NULL);
    world = new VDTile*[worldSizeX]();
    for (int x = 0; x < worldSizeX; x++) {
        world[x] = new VDTile[worldSizeY]();
    };
    for (int x = 0; x < worldSizeX; x++) {
        for (int y = 0; y < worldSizeY; y++) {
            readFile = ReadFile(file, &world[x][y], sizeof(world[x][y]), &numberOfBytesRead, NULL);
            if (readFile) {
            };
        };
    };
    closeFile = CloseHandle(file);
    if (closeFile == 0) {
        VDError("Failed to close the world file.", __FILE__, __LINE__);
    };
};



/* **********************************************************************************************************************
   WriteNPCs
   ---------
   Saves the NPC's to hard disk.
   #param  fileName  The name of the NPC file to load.
   **********************************************************************************************************************/

void VDWorld::WriteNPCs(LPCSTR fileName) {
    DWORD numberOfBytesToWrite = 0;
    DWORD numberOfBytesWritten = 0;
    HANDLE file = NULL;
    BOOL writeFile = FALSE;
    BOOL closeFile = FALSE;

    file = CreateFile(fileName, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

    // make sure we let it know how many NPC elements we're adding
    numberOfBytesToWrite = npcs.size();
    writeFile = WriteFile(file, &numberOfBytesToWrite, sizeof(numberOfBytesToWrite), &numberOfBytesWritten, NULL);

    // write all the NPC's
    for (list<VDNPC>::iterator anNPC = npcs.begin(); anNPC != npcs.end();) {
        writeFile = WriteFile(file, &anNPC, sizeof(anNPC), &numberOfBytesWritten, NULL);
    };

    closeFile = CloseHandle(file);
};



/* **********************************************************************************************************************
   LoadNPCs
   --------
   Loads the NPC's into the game.
   #param  fileName  The name of the NPC file to load.
   **********************************************************************************************************************/

void VDWorld::LoadNPCs(LPCSTR fileName) {
    DWORD numberOfNPCs = 0;
    DWORD numberOfBytesRead = 0;
    HANDLE file = NULL;
    BOOL readFile = FALSE;
    BOOL closeFile = FALSE;

    // load file
    file = CreateFile(fileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if (file == INVALID_HANDLE_VALUE) {
        VDError("Failed to load the world file.", __FILE__, __LINE__);
    };

    readFile = ReadFile(file, &numberOfNPCs, sizeof(numberOfNPCs), &numberOfBytesRead, NULL);
    while (numberOfNPCs > 0) {
        VDNPC newNPC;
        readFile = ReadFile(file, &newNPC, sizeof(newNPC), &numberOfBytesRead, NULL);
        npcs.push_back(newNPC);
        numberOfNPCs--;
    };

    closeFile = CloseHandle(file);
    if (closeFile == 0) {
        VDError("Failed to close the world file.", __FILE__, __LINE__);
    };
};



/* **********************************************************************************************************************
   ReallocateWorld
   ---------------
   Reallocates the map given the new coordinates.
   #param  newWorldLocationX  X coordinate attained from mouse position.
   #param  newWorldLocationY  Y coordinate attained from mouse position.
   #return  The method in which we reallocated the map (1 = shifted right, 2 = shifted down, 3 = shifted right + down)
   **********************************************************************************************************************/

int VDWorld::ReallocateWorld(int newWorldLocationX, int newWorldLocationY) {
    int method = 0;
    int oldWorldSizeX = 0, oldWorldSizeY = 0;

    // copy world
    VDTile **oldWorld;
    oldWorldSizeX = worldSizeX;
    oldWorldSizeY = worldSizeY;
    oldWorld = new VDTile*[oldWorldSizeX]();
    for (int x = 0; x < oldWorldSizeX; x++) {
        oldWorld[x] = new VDTile[oldWorldSizeY]();
    };
    for (int x = 0; x < oldWorldSizeX; x++) {
        for (int y = 0; y < oldWorldSizeY; y++) {
            oldWorld[x][y].SetStatus(world[x][y].GetStatus());
            oldWorld[x][y].SetImageFileLocation(world[x][y].GetImageFileLocation());
            //oldWorld[x][y].SetSpawns(world[x][y].GetSpawns());
        };
    };

    // deallocate the world
    for (int x = 0; x < worldSizeX; x++) {
        delete[] world[x];
        world[x] = NULL;
    };
    delete[] world;
    world = NULL;

    // see how we're going to allocate the new world
    if (newWorldLocationX < 0 && newWorldLocationY >= 0) {
        // old tiles shift to the right
        method = 1;
        worldSizeX -= newWorldLocationX;
        if (newWorldLocationY >= worldSizeY) {
            worldSizeY = newWorldLocationY + 1;
        };
    } else if (newWorldLocationY < 0 && newWorldLocationX >= 0) {
        // old tiles shift down
        method = 2;
        worldSizeY -= newWorldLocationY;
        if (newWorldLocationX >= worldSizeX) {
            worldSizeX = newWorldLocationX + 1;
        };
    } else if (newWorldLocationX < 0 && newWorldLocationY < 0) {
        // old tiles shift down and to the right
        method = 3;
        worldSizeX -= newWorldLocationX;
        worldSizeY -= newWorldLocationY;
    } else {
        // old tiles stay where they are
        method = 4;
        if (newWorldLocationX >= worldSizeX) {
            worldSizeX = newWorldLocationX + 1;
        };
        if (newWorldLocationY >= worldSizeY) {
            worldSizeY = newWorldLocationY + 1;
        };
    };

    // allocate the new world tiles
    world = new VDTile*[worldSizeX]();
    for (int x = 0; x < worldSizeX; x++) {
        world[x] = new VDTile[worldSizeY]();
    };

    // populate the new world with the old world tiles
    for (int x = 0; x < oldWorldSizeX; x++) {
        for (int y = 0; y < oldWorldSizeY; y++) {
            switch (method) {
                case 1:
                    world[x - newWorldLocationX][y].SetStatus(oldWorld[x][y].GetStatus());
                    world[x - newWorldLocationX][y].SetImageFileLocation(oldWorld[x][y].GetImageFileLocation());
                    //world[x - newWorldLocationX][y].SetSpawns(oldWorld[x][y].GetSpawns());
                    break;
                case 2:
                    world[x][y - newWorldLocationY].SetStatus(oldWorld[x][y].GetStatus());
                    world[x][y - newWorldLocationY].SetImageFileLocation(oldWorld[x][y].GetImageFileLocation());
                    //world[x][y - newWorldLocationY].SetSpawns(oldWorld[x][y].GetSpawns());
                    break;
                case 3:
                    world[x - newWorldLocationX][y - newWorldLocationY].SetStatus(oldWorld[x][y].GetStatus());
                    world[x - newWorldLocationX][y - newWorldLocationY].SetImageFileLocation(oldWorld[x][y].GetImageFileLocation());
                    //world[x - newWorldLocationX][y - newWorldLocationY].SetSpawns(oldWorld[x][y].GetSpawns());
                    break;
                case 4:
                    world[x][y].SetStatus(oldWorld[x][y].GetStatus());
                    world[x][y].SetImageFileLocation(oldWorld[x][y].GetImageFileLocation());
                    //world[x][y].SetSpawns(oldWorld[x][y].GetSpawns());
                    break;
                default:
                    break;
            };
        };
    };

    // deallocate the old world
    for (int x = 0; x < oldWorldSizeX; x++) {
        delete[] oldWorld[x];
        oldWorld[x] = NULL;
    };
    delete[] oldWorld;
    oldWorld = NULL;

    return method;
};



/* **********************************************************************************************************************
   CheckNewLocation
   ----------------
   Looks at the status of the tile in the given direction.
   #param  direction        Direction of adjacent tile.
   #param  entityLocationX  Entity's x coordinate.
   #param  entityLocationY  Entity's y coordinate.
   #return  The status of the tile.
   **********************************************************************************************************************/

int VDWorld::CheckNewLocation(int direction, int entityLocationX, int entityLocationY) {
    switch (direction) {
        // checking if spot is valid
        case 0:
            if (entityLocationX < 0 || entityLocationX > worldSizeX - 1 || entityLocationY < 0 || entityLocationY > worldSizeY - 1) {
                return NA;
            } else {
                return world[entityLocationX][entityLocationY].GetStatus();
            };
            break;
        // left
        case 1:
            if (entityLocationX > 0) {
                return world[entityLocationX - 1][entityLocationY].GetStatus();
            } else {
                return NA;
            };
            break;
        // right
        case 2:
            if (entityLocationX < worldSizeX - 1) {
                return world[entityLocationX + 1][entityLocationY].GetStatus();
            } else {
                return NA;
            };
            break;
        // up
        case 3:
            if (entityLocationY > 0) {
                return world[entityLocationX][entityLocationY - 1].GetStatus();
            } else {
                return NA;
            };
            break;
        // down
        case 4:
            if (entityLocationY < worldSizeY - 1) {
                return world[entityLocationX][entityLocationY + 1].GetStatus();
            } else {
                return NA;
            };
            break;
        // N/A
        default:
            return NA;
            break;
    };
};



/* **********************************************************************************************************************
   AddNPC
   ------
   Adds an NPC to the list.
   #param  name               Name of the NPC.
   #param  imageFileLocation  The file location of the sprite.
   #return  TRUE if successful, FALSE if NPC already exists
   **********************************************************************************************************************/

int VDWorld::AddNPC(LPCSTR name, LPCSTR imageFileLocation) {
    // see if we're trying to add an NPC that already exists
    for (list<VDNPC>::iterator anNPC = npcs.begin(); anNPC != npcs.end(); ++anNPC) {
        if (strcmp(name, anNPC->GetName()) == 0) {
            // change the image, keep the name
            anNPC->SetImageFileLocation(imageFileLocation);

            return FALSE;
        };
    };

    // add the NPC
    VDNPC newNPC = VDNPC(name);
    newNPC.SetImageFileLocation(imageFileLocation);
    npcs.push_back(newNPC);

    return TRUE;
};



/* **********************************************************************************************************************
   RemoveNPC
   ---------
   Removes an NPC from the list given their name.
   #param  name  Name of the NPC.
   **********************************************************************************************************************/

void VDWorld::RemoveNPC(LPCSTR name) {
    for (list<VDNPC>::iterator anNPC = npcs.begin(); anNPC != npcs.end();) {
        if (strcmp(name, anNPC->GetName()) == 0) {
            anNPC = npcs.erase(anNPC);
        } else {
            ++anNPC;
        };
    };
};



/* **********************************************************************************************************************
   AddNPCDialogue
   --------------
   Adds a dialogue to the NPC's list
   #param  name  Name of the NPC.
   #param  id    The ID of the dialogue.
   #param  text  The text of the dialogue.
   #return  TRUE if successful, FALSE if the dialogue ID already exists or if the NPC doesn't exist.
   **********************************************************************************************************************/

int VDWorld::AddNPCDialogue(LPCSTR name, int id, LPCSTR text) {
    // look for the NPC
    for (list<VDNPC>::iterator anNPC = npcs.begin(); anNPC != npcs.end(); ++anNPC) {
        // got him, add the dialogue
        if (strcmp(name, anNPC->GetName()) == 0) {
            return anNPC->AddDialogue(id, text);
        };
    };

    return FALSE;
};

void VDWorld::RemoveNPCDialogue(LPCSTR name, int id) {
    // look for the NPC
    for (list<VDNPC>::iterator anNPC = npcs.begin(); anNPC != npcs.end(); ++anNPC) {
        // got him, add the dialogue
        if (strcmp(name, anNPC->GetName()) == 0) {
            anNPC->RemoveDialogue(id);
        };
    };
};

LPCSTR VDWorld::GetNPCDialogueText(LPCSTR npcName, int index) {
    for (list<VDNPC>::iterator anNPC = npcs.begin(); anNPC != npcs.end(); ++anNPC) {
        if (strcmp(npcName, anNPC->GetName()) == 0) {
            return anNPC->GetDialogue(index);
        };
    };

    return NULL;
};

LPCSTR VDWorld::GetNPCDialogueTextFromID(LPCSTR npcName, int id) {
    for (list<VDNPC>::iterator anNPC = npcs.begin(); anNPC != npcs.end(); ++anNPC) {
        if (strcmp(npcName, anNPC->GetName()) == 0) {
            return anNPC->GetDialogueFromID(id);
        };
    };

    return NULL;
};

unsigned int VDWorld::GetNPCDialogueTextID(LPCSTR npcName, int index) {
    for (list<VDNPC>::iterator anNPC = npcs.begin(); anNPC != npcs.end(); ++anNPC) {
        if (strcmp(npcName, anNPC->GetName()) == 0) {
            return anNPC->GetDialogueID(index);
        };
    };

    return NULL;
};



/* **********************************************************************************************************************
   Accessors and Modifiers
   **********************************************************************************************************************/

VDTile VDWorld::GetTile(int x, int y) {
    return world[x][y];
};

LPCTSTR VDWorld::GetTileImageFileLocation(int x, int y) {
    return world[x][y].GetImageFileLocation();
};
void VDWorld::SetTileStatus(int x, int y, int theStatus) {
    world[x][y].SetStatus(theStatus);
};

void VDWorld::SetTileImageFileLocation(int x, int y, LPCSTR theImageFileLocation) {
    world[x][y].SetImageFileLocation(theImageFileLocation);
};

/*void VDWorld::SetTileSpawns(int x, int y, list<VDSpawn> theSpawns) {
    world[x][y].SetSpawns(theSpawns);
};*/

int VDWorld::GetWorldSizeX() {
    return worldSizeX;
};

int VDWorld::GetWorldSizeY() {
    return worldSizeY;
};

list<VDNPC> VDWorld::GetNPCs() {
    return npcs;
};

VDNPC VDWorld::GetNPC(LPCSTR name) {
    for (list<VDNPC>::iterator anNPC = npcs.begin(); anNPC != npcs.end(); ++anNPC) {
        VDNPC npc = *anNPC;
        if (strcmp(name, anNPC->GetName()) == 0) {
            return *anNPC;
        };
    };

    return NULL;
};